@@ -28,8 +28,7 @@ import time
 from oslo_utils import excutils
 from oslo_utils import netutils
 import requests
-import six
-import six.moves.urllib.parse as urlparse
+import urllib.parse as urlparse
 from urllib3 import connection as httplib
 
 from oslo_vmware._i18n import _
@@ -135,7 +134,7 @@ class FileHandle(object):
                                self._build_vim_cookie_header(cookies)})
             if content_type:
                 headers.update({'Content-Type': content_type})
-            for key, value in six.iteritems(headers):
+            for key, value in headers.items():
                 conn.putheader(key, value)
             conn.endheaders()
             return conn
@@ -354,7 +353,13 @@ class VmdkHandle(FileHandle):
         LOG.debug("Lease for %(url)s is in state: %(state)s.",
                   {'url': self._url,
                    'state': state})
-        if state == 'ready':
+        if self._get_progress() < 100:
+            LOG.error("Aborting lease for %s due to incomplete transfer.",
+                      self._url)
+            self._session.invoke_api(self._session.vim,
+                                     'HttpNfcLeaseAbort',
+                                     self._lease)
+        elif state == 'ready':
             LOG.debug("Releasing lease for %s.", self._url)
             self._session.invoke_api(self._session.vim,
                                      'HttpNfcLeaseComplete',
@@ -507,7 +512,14 @@ class VmdkWriteHandle(VmdkHandle):
         super(VmdkWriteHandle, self).__init__(session, lease, url, self._conn)
 
     def get_imported_vm(self):
-        """"Get managed object reference of the VM created for import."""
+        """"Get managed object reference of the VM created for import.
+
+        :raises: VimException
+        """
+        if self._get_progress() < 100:
+            excep_msg = _("Incomplete VMDK upload to %s.") % self._url
+            LOG.exception(excep_msg)
+            raise exceptions.ImageTransferException(excep_msg)
         return self._vm_ref
 
     def tell(self):
